﻿// ScoreManager.cs
//
// @idex Unity5.5.0f1 / MonoDevelop5.9.6
// @auth GGJFKS17C.FCEI.No-Va
// @date 2017/01/21 
//------------------------------------------------------------------------------------------------------------------------
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

//----------------------------------------------------------------------------------------------------
// スコアを管理する 
//----------------------------------------------------------------------------------------------------
public class ScoreManager : MonoBehaviour
{
	//--------------------------------------------------------------------------------
	// メンバ変数 
	//--------------------------------------------------------------------------------
	static ScoreManager instance;	// インスタンス 

	const float COMBO_TIME_LIMIT = 3.5f;	// コンボが継続する時間 
	const float COMBO_RATE       = 1.25f;	// コンボ倍率 

	float comboTimer;	// コンボが継続を判定するタイマー  
	int   combo;		// コンボ数 

	int score;		// スコア 
	int viewScore;	// 表示上のスコア 
	int tempScore;	// 加算される前のストックスコア 

	public static float comboTimerValue{ get{ return COMBO_TIME_LIMIT-instance.comboTimer; } }
	public static int comboValue{ get{ return instance.combo; } }
	public static int scoreValue{ get{ return instance.score; } }



	//--------------------------------------------------------------------------------
	// コンストラクタ 
	//--------------------------------------------------------------------------------
	void Awake ()
	{
		// シングルトン生成 (Fast-Priority型) 
		if(instance == null){
			instance = this;
			DontDestroyOnLoad(this.gameObject);
		}
		else{
			GameObject.Destroy(this.gameObject);
		}
	}

	//--------------------------------------------------------------------------------
	// スコア加算 
	//--------------------------------------------------------------------------------
	public static void AddScore (int val)
	{
		instance.AddScoreNaked(val);
	}
	void AddScoreNaked (int val)
	{
		// コンボ加算 
		combo++;
		StopCoroutine("ComboWait");
		StartCoroutine("ComboWait");

		// コンボを加味してスコアを加算  
		int addScore = val;
		for(int i=1; i<instance.combo; i++){
			addScore = (int)((float)addScore * COMBO_RATE);
		}

		instance.tempScore += addScore;
	}
		
	//--------------------------------------------------------------------------------
	// コンボ待ち  
	//--------------------------------------------------------------------------------
	IEnumerator ComboWait ()
	{
		comboTimer = 0;
		while(comboTimer < COMBO_TIME_LIMIT){
			yield return null;
			comboTimer += Time.deltaTime;
		}

		score += tempScore;
		UpdateScoreView();
		SoundPlayer.PlaySE(SoundPlayer.SE.SCORE);

		tempScore = 0;
		combo = 0;
	}

	//--------------------------------------------------------------------------------
	// 表示更新 
	//--------------------------------------------------------------------------------
	void UpdateScoreView ()
	{
		StopCoroutine("UpdateScoreViewCoroutine");
		StartCoroutine("UpdateScoreViewCoroutine");
	}
	IEnumerator UpdateScoreViewCoroutine ()
	{
		const float MAX_TIME = 0.5f;	// アニメーションにかける時間 

		int fromScore = viewScore;
		int toScore   = score;

		float time = 0;
		while(time < MAX_TIME){
			viewScore = (int)((fromScore*(MAX_TIME-time) + toScore*(time)) / MAX_TIME);

			yield return null;
			time += Time.deltaTime;
		}
		viewScore = toScore;
	}

	//--------------------------------------------------------------------------------
	// スコアのリセット 
	//--------------------------------------------------------------------------------
	public static void Reset ()
	{
		instance.comboTimer=0;
		instance.combo=0;
		instance.score=0;
		instance.viewScore=0;
		instance.tempScore=0;
	}

	//--------------------------------------------------------------------------------
	// コンボ切り 
	//--------------------------------------------------------------------------------
	public static void ComboOff ()
	{
		instance.combo = 0;
		instance.StopCoroutine("ComboWait");
	}

	//--------------------------------------------------------------------------------
	// エフェクト 
	//--------------------------------------------------------------------------------
	void Update ()
	{
		if(combo >= 3){
			SpeedFx.Spawn();
		}
	}

	//--------------------------------------------------------------------------------
	// debug 
	//--------------------------------------------------------------------------------
	//void OnGUI ()
	//{
	//	if(DebugSwitch.isDebugEnable == false){ return; }
	//
	//	GUILayout.Label("Score:"+score);
	//	GUILayout.Label("View:"+viewScore);
	//	GUILayout.Label("Temp:"+tempScore);
	//	GUILayout.Label("combo+"+combo);
	//
	//	if(GUILayout.Button("jump")){
	//		AddScore(1000);
	//	}
	//}
}
